﻿using ProjectElectricLocomotive.DrawingObjects;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectElectricLocomotive
{
    public partial class FormLocomotiveConfig : Form
    {

        /// <summary>
        /// Переменная-выбранный локомотив
        /// </summary>
        DrawingLocomotive? _loco = null;

        /// <summary>
        /// Событие
        /// </summary>

        private event Action<DrawingLocomotive>? EventAddLoco;

        public FormLocomotiveConfig()
        {
            InitializeComponent();

            panelPastelRed.MouseDown += PanelColor_MouseDown;
            panelPastelPink.MouseDown += PanelColor_MouseDown;
            panelPastelOrange.MouseDown += PanelColor_MouseDown;
            panelPastelYellow.MouseDown += PanelColor_MouseDown;
            panelSkyBlue.MouseDown += PanelColor_MouseDown;
            panelPastelGreen.MouseDown += PanelColor_MouseDown;
            panelPastelLilac.MouseDown += PanelColor_MouseDown;
            panelPastelViolet.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (sender, e) => Close();
        }

        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingLocomotive> ev)
        {
            if (EventAddLoco == null)
            {
                EventAddLoco = ev;
            }
            else
            {
                EventAddLoco += ev;
            }
        }

        /// <summary>
        /// Отрисовать Loco
        /// </summary>
        private void DrawLoco()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _loco?.SetPosition(10, 10);
            _loco?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"><labelSimpleObject/param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
           DragDropEffects.Move | DragDropEffects.Copy);
        }

        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _loco = new DrawingLocomotive(
                        (int)numericUpDownSpeed.Value,
                        (int)numericUpDownWeight.Value,
                        Color.White,
                        pictureBoxObject.Width,
                        pictureBoxObject.Height
                        );
                    break;
                case "labelModifiedObject":
                    _loco = new DrawingElectricLocomotive(
                        (int)numericUpDownSpeed.Value,
                        (int)numericUpDownWeight.Value,
                        Color.White,
                        Color.Black,
                        checkBoxHorns.Checked,
                        checkBoxSeifBatteries.Checked,
                        pictureBoxObject.Width,
                        pictureBoxObject.Height
                        );
                    break;
            }
            DrawLoco();
        }


        // НЕ УВЕРЕНА, ЧТО ВЕРНО, драгдроп для цветов
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
           DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            ((Label)sender).BackColor = (Color)e.Data.GetData(typeof(Color));
            switch (((Label)sender).Name)
            {
                case "labelColor":
                    _loco.SetColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAddColor":
                    if (_loco is not DrawingLocomotive) return;
                    else
                    {
                        (_loco as DrawingElectricLocomotive).SetAddColor((Color)e.Data.GetData(typeof(Color)));
                    }
                    break;
            }
            DrawLoco();
        }
       
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Добавление loco
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddLoco?.Invoke(_loco);
            Close();
        }
    }
}
