﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectElectricLocomotive.DrawingObjects;
using ProjectElectricLocomotive.MovementStrategy;

namespace ProjectElectricLocomotive.Generics
{
    internal class LocomotiveGenericCollection<T, U> where T : DrawingLocomotive where U : IMoveableObject
    {
        /// <summary>
        /// Получение объектов коллекции
        /// </summary>
        public IEnumerable<T?> GetLocomotives => _collection.GetLocomotives();


        //ширина/высота окна
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        //ширина/высота занимаемого места
        private readonly int _placeSizeWidth = 85;
        private readonly int _placeSizeHeight = 50;

        /// Набор объектов
        private readonly SetGeneric<T> _collection;

        public LocomotiveGenericCollection(int picWidth, int picHeight)
        {
            //  немного странная логика, что-то я пока ее не особо понимаю, зачем нам ААААА дошло...
            //  высчитываем размер массива для setgeneric
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }

        /// Перегрузка оператора сложения
        public static int operator +(LocomotiveGenericCollection<T, U> collect, T? loco)
        {
            if (loco == null)
            {
                return -1;
            }
            return collect._collection.Insert(loco);
        }

        /// Перегрузка оператора вычитания
        public static T? operator -(LocomotiveGenericCollection<T, U> collect, int pos)
        {
            T? obj = collect._collection[pos];
            collect._collection.Remove(pos);
            return obj;
        }

        // получение объекта imoveableObj
        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }

        /// Вывод всего набора объектов
        public Bitmap ShowLocomotives()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {
                    //линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }

        private void DrawObjects(Graphics g)
        {
            int HeightObjCount = _pictureHeight / _placeSizeHeight;
            for (int i = 0; i < _collection.Count; i++)
            {
                T? type = _collection[i];
                if (type != null)
                {
                    type.SetPosition(
                        (int)(i / HeightObjCount * _placeSizeWidth),
                        (HeightObjCount - 1) * _placeSizeHeight - (int)(i % HeightObjCount * _placeSizeHeight));
                    type?.DrawTransport(g);
                }
            }
        }
    }
}
